##############################################################################
# author       # TecDroiD
# date         # 2022-12-02
# ---------------------------------------------------------------------------
# description  # simple python project management tool
#              #
#
#
##############################################################################
from upm.template import Template
import os
import logging

def get_template_file(cfg, filename):
    base = ''
    if 'template_path' in cfg:
        base = os.path.expanduser(cfg['template_path'])

    return os.path.join(base, f'{filename}.tmpl')


def get_values(plist):
    ''' get values from parameter list (-p)
    '''
    retval={}
    if isinstance(plist, list):
        for item in plist:
            k,v = item.split('=', maxsplit=1)
            retval[k] = v
    return retval


def scan(cfg, args):
    ''' scan directory
    '''
    basedir = args.basedir
    name = args.name
    desc = args.description
    tmpl = Template.scan(name, basedir, desc)
    filename = get_template_file(cfg, name)

    if args.no_write:
        print(f'would write file {filename}:')
        print(tmpl.json)
    else:
        print(f'Writing file {filename}.')
        with open(filename, 'w') as fp:
            fp.write(tmpl.json)

def create(cfg, args):
    ''' create a filestructure from pattern
    '''
    item = args.item
    dest = args.destination

    # create destination directory if not existent
    if not os.path.exists(dest):
        os.makedirs(dest)

    # replace pattern or not.. that's the question
    rpl = True
    if args.no_replace is True:
        rpl = False

    pattern = cfg['pattern']
    pattern.update(get_values(args.parameter))

    tpath = get_template_file(cfg, item)
    template = Template.from_file(tpath)
    template.create(dest, pattern, _replace=rpl)


def do_list(cfg, args):
    ''' list known templates
    '''
    print('Known templates:\n')
    for file in os.listdir(os.path.expanduser(cfg['template_path'])):
        if file.endswith('.tmpl'):
            print (file.replace('.tmpl',''))
    print ('\nuse upm describe <template> for more info on a template')


def describe(cfg, args):
    ''' describe a single template
    '''
    item = args.item
    tpath = get_template_file(cfg, item)
    template = Template.from_file(tpath)
    print(template)


def run (cfg, args):
    ''' run the upm command
    '''
    logging.debug(f'Running action {args.action}')
    actions= {
            'scan' : scan,
            'init' : create,
            'list' : do_list,
            'desc' : describe,
            }
    actions[args.action](cfg, args)




# -*- coding: utf-8 -*-
###############################################################################
# author       # TecDroiD
# date         # 2022-12-02
# ---------------------------------------------------------------------------
# description  # a Template for files or directories
#              #
#              #
##############################################################################
import json
import base64
import os
import logging


def pattern_replace(text, pattern, pre='{{', post='}}'):
    ''' replace text with pattern if possible
    '''
    haystack = text
    for key,value in pattern.items():
        logging.debug(f'replacing {key} with {value}')
        haystack = haystack.replace(pre+key+post, value)
    return haystack

class Template():
    ''' this is a filesystem template for upm
    '''
    def from_file(filename):
        ''' read a json file and create a template from it
        '''
        name = os.path.basename(filename)
        with open(filename) as fp:
            template = Template('')
            template.values = json.load(fp)
            return template
        return None

    def scan(name, path, description='', _cpath='', _tmpl=None):
        ''' scan a directory to create a template
        '''
        logging.debug(f'scanning path {path}/{_cpath}')
        template = _tmpl if _tmpl is not None else Template(name, description)
        for file in os.listdir(os.path.join(path,_cpath)):
            # get all files in directory
            relpath = os.path.join(_cpath, file)
            filepath = os.path.join(path, relpath)
            if os.path.isdir(filepath):
                # scan subdirs
                Template.scan(name, path, description, relpath, template)
            else:
                #read file and append data
                with open(filepath, 'rb') as fp:
                    content = fp.read()
                    logging.debug(f'creating file {path}/{_cpath}/{file}')
                    template.add_item(_cpath, file, content)
        return template

    def __init__(self, name, description=''):
        ''' initialize a template
        '''
        self.values = {}
        self.values['name'] = name
        self.values['description'] = description
        self.values['files'] = {}

    def add_item(self, path, name, content=''):
        ''' create a directory path in files structure
        '''
        files = self.files
        if path != '':
            directory = path.split('/')
            # create directory struture if neccessary
            for subdir in directory:
                if not subdir in files:
                    files[subdir] = {}
                files = files[subdir]
        #create the file
        files[name] = base64.b64encode(content).decode('utf8')

    def write_content(self, filename, content, pattern, replace=True):
        ''' write file content
        '''
        # entry is file
        with open(filename, 'wb') as fp:
            logging.debug(f'filecontent {content}')
            bcontent = base64.b64decode(content.encode('utf8')).decode()
            if replace is True:
                bcontent = pattern_replace(bcontent, pattern)
            # write file
            fp.write(bcontent.encode('utf8'))

    def create(
               self,
               basepath,
               pattern={},
               _cpath='',
               _files=None,
               _replace=True ):
        ''' create the template file structure using the patterns
        for pattern replacement
        '''
        files = _files if _files is not None else self.files

        logging.debug(f'---- iteratinging subpath {_cpath}')
        for name,entry in files.items():
            logging.debug(f'** {_cpath} : having {name}')
            # replace
            nname = pattern_replace(name, pattern, '', '')
            relpath = os.path.join(_cpath, nname)
            abspath = os.path.join(basepath, relpath)
            if isinstance(entry, dict):
                # entry is a directory, create subdir
                logging.debug(f'creating directory {abspath}')
                if not os.path.exists(abspath):
                    os.mkdir(abspath)
                logging.debug(f'---- iterating into directory {relpath}')
                self.create(basepath, pattern, relpath, entry, _replace)
            else:
                # entry is a file, create and write
                # create filename entry for internal replacement
                pattern['current_file'] = entry
                # and write file
                logging.debug(f'creating file  {abspath}')
                self.write_content(abspath, entry, pattern, _replace)
         # done - hopefully..

    def dump(self, filename):
        ''' write yourself to a file
        '''
        with open(filename,'w') as fp:
            fp.write(self.json)

    def __repr__(self):
        desc = self.description
        if len(desc) == 0:
            desc ='<NO DESCRIPTION>'
        return f'Template {self.name}\n{desc}'

    @property
    def json(self):
        ''' dump the template to json
        '''
        return json.dumps(self.values, indent=2)

    @property
    def files(self):
        ''' get the files object of the template
        '''
        return self.values['files']

    @property
    def name(self):
        return self.values['name']

    @property
    def description(self):
        return self.values['description']





#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
# author       # TecDroiD
# date         # 2022-12-02
# ---------------------------------------------------------------------------
# description  # simple python project management tool - cli
#              #
#              #
##############################################################################
import sys
from argparse import ArgumentParser
import logging

from upm import config
from upm import upm

def add_scan_parser(subs):
    ''' create subparser to describe an item
    '''
    parser = subs.add_parser('scan', help='create new template from directory')
    parser.add_argument(
            '-n', '--no-write',
            help='do not write template file',
            action='store_true'
            )
    parser.add_argument(
            '-d', '--description',
            help='optional template description',
            default=''
            )
    parser.add_argument('basedir', help='directory to scan')
    parser.add_argument('name', help='name of the new template')


def add_describe_parser(subs):
    ''' create subparser to describe an item
    '''
    parser = subs.add_parser('desc', help='describe item')
    parser.add_argument('item', help='Item to describe')


def add_init_parser(subs):
    ''' create a subparser for initializing items
    '''
    parser = subs.add_parser('init', help='initialize an item')
    parser.add_argument(
            '-p', '--parameter',
            nargs='*',
            help='parameters in form key=value'
            )
    parser.add_argument(
            '-n', '--no-replace',
            action='store_true',
            help='Do not expand pattern for enhancing existing templates'
            )
    parser.add_argument('item', help='Item to initialize')
    parser.add_argument(
            'destination',
            help='destination of the new item'
            )


def parse_arguments():
    ''' creates an argument parser and parses arguments
    '''
    parser = ArgumentParser(
            description='helps you generating python projects',
            )
    parser.add_argument(
                        '-v', '--verbose',
                        action='store_true',
                        help='tell me more'
                        )
    subs = parser.add_subparsers(
                        dest='action',
                        help='subfunctions'
                        )
    subs.add_parser('list', help='list possible elements')
    add_describe_parser(subs)
    add_init_parser(subs)
    add_scan_parser(subs)
    return parser.parse_args()


def main():
    ''' this is the main function who controls all the other things
    '''
    args = parse_arguments()
    level = logging.DEBUG if args.verbose else logging.INFO
    logging.basicConfig(level=level)

    # load configuration
    cfg = config.load(['~/.upm/config','./.upm'])
    # do
    upm.run(cfg, args)

if __name__ == '__main__':
    sys.exit(main())



# -*- coding: utf-8 -*-
###############################################################################
# author       # TecDroiD
# date         # 2022-12-02
# ---------------------------------------------------------------------------
# description  # configuration file management
#              #
#              #
##############################################################################
import json
import os
import logging
from datetime import date

BASECONFIG = {
    'pattern' : {
        'current_date' : date.today().isoformat(),
        'current_year' : str(date.today().year)
        },
    'template_path' : 'templates'
}


def synchronize(config, overlay):
    ''' TODO: synchronize config with overlay recursively
    '''
    for k, v in overlay.items():
        if k not in config or not isinstance(v, dict):
            config[k] = v
        else:
            synchronize(config[k], v)

    return config


def load(files=['./.cfg']):
    ''' load configuration files
    '''
    cfg = dict(BASECONFIG)

    for file in files:
        logging.debug(f'trying config file {file}')
        fn = os.path.expanduser(file)
        if os.path.exists(fn):
            with open(fn) as fp:
                logging.debug(f'reading config {file}')

                cfg = synchronize(cfg, json.load(fp))

    logging.debug(f'{cfg}')

    return cfg
